# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: openocd.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='openocd.proto',
  package='rpi',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\ropenocd.proto\x12\x03rpi\"$\n\x10LogStreamRequest\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\"!\n\x11LogStreamResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\x1e\n\x0eProgramRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x06\n\x04void2\x8a\x02\n\x07OpenOcd\x12@\n\rProgramDevice\x12\x13.rpi.ProgramRequest\x1a\x16.rpi.LogStreamResponse(\x01\x30\x01\x12\x32\n\x0bResetDevice\x12\t.rpi.void\x1a\x16.rpi.LogStreamResponse0\x01\x12\"\n\nStartDebug\x12\t.rpi.void\x1a\t.rpi.void\x12!\n\tStopDebug\x12\t.rpi.void\x1a\t.rpi.void\x12\x42\n\x0fLogStreamCreate\x12\x15.rpi.LogStreamRequest\x1a\x16.rpi.LogStreamResponse0\x01\x62\x06proto3'
)




_LOGSTREAMREQUEST = _descriptor.Descriptor(
  name='LogStreamRequest',
  full_name='rpi.LogStreamRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='rpi.LogStreamRequest.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=22,
  serialized_end=58,
)


_LOGSTREAMRESPONSE = _descriptor.Descriptor(
  name='LogStreamResponse',
  full_name='rpi.LogStreamResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='rpi.LogStreamResponse.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=93,
)


_PROGRAMREQUEST = _descriptor.Descriptor(
  name='ProgramRequest',
  full_name='rpi.ProgramRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='rpi.ProgramRequest.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=95,
  serialized_end=125,
)


_VOID = _descriptor.Descriptor(
  name='void',
  full_name='rpi.void',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=133,
)

DESCRIPTOR.message_types_by_name['LogStreamRequest'] = _LOGSTREAMREQUEST
DESCRIPTOR.message_types_by_name['LogStreamResponse'] = _LOGSTREAMRESPONSE
DESCRIPTOR.message_types_by_name['ProgramRequest'] = _PROGRAMREQUEST
DESCRIPTOR.message_types_by_name['void'] = _VOID
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogStreamRequest = _reflection.GeneratedProtocolMessageType('LogStreamRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOGSTREAMREQUEST,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.LogStreamRequest)
  })
_sym_db.RegisterMessage(LogStreamRequest)

LogStreamResponse = _reflection.GeneratedProtocolMessageType('LogStreamResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOGSTREAMRESPONSE,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.LogStreamResponse)
  })
_sym_db.RegisterMessage(LogStreamResponse)

ProgramRequest = _reflection.GeneratedProtocolMessageType('ProgramRequest', (_message.Message,), {
  'DESCRIPTOR' : _PROGRAMREQUEST,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.ProgramRequest)
  })
_sym_db.RegisterMessage(ProgramRequest)

void = _reflection.GeneratedProtocolMessageType('void', (_message.Message,), {
  'DESCRIPTOR' : _VOID,
  '__module__' : 'openocd_pb2'
  # @@protoc_insertion_point(class_scope:rpi.void)
  })
_sym_db.RegisterMessage(void)



_OPENOCD = _descriptor.ServiceDescriptor(
  name='OpenOcd',
  full_name='rpi.OpenOcd',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=136,
  serialized_end=402,
  methods=[
  _descriptor.MethodDescriptor(
    name='ProgramDevice',
    full_name='rpi.OpenOcd.ProgramDevice',
    index=0,
    containing_service=None,
    input_type=_PROGRAMREQUEST,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ResetDevice',
    full_name='rpi.OpenOcd.ResetDevice',
    index=1,
    containing_service=None,
    input_type=_VOID,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartDebug',
    full_name='rpi.OpenOcd.StartDebug',
    index=2,
    containing_service=None,
    input_type=_VOID,
    output_type=_VOID,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopDebug',
    full_name='rpi.OpenOcd.StopDebug',
    index=3,
    containing_service=None,
    input_type=_VOID,
    output_type=_VOID,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LogStreamCreate',
    full_name='rpi.OpenOcd.LogStreamCreate',
    index=4,
    containing_service=None,
    input_type=_LOGSTREAMREQUEST,
    output_type=_LOGSTREAMRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_OPENOCD)

DESCRIPTOR.services_by_name['OpenOcd'] = _OPENOCD

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

import argparse
import grpc
import logging
import threading
import tempfile
from pathlib import Path
from configparser import ConfigParser
from concurrent import futures

import oocd_tool._credentials as _credentials
import oocd_tool.openocd_pb2 as openocd_pb2
import oocd_tool.openocd_pb2_grpc as openocd_pb2_grpc
from oocd_tool.rpc_impl import *

_LOGGER = logging.getLogger(__name__)
class OpenOcd(openocd_pb2_grpc.OpenOcdServicer):

    def __init__(self, config):
        super(OpenOcd, self).__init__()
        self.config = config

    def LogStreamCreate(self, request, context):
        _LOGGER.info("LogStreamCreate called.")
        stop_event = threading.Event()
        self.log_reader = LogReader()

        def on_rpc_done():
            _LOGGER.debug("Attempting to regain servicer thread.")
            stop_event.set()
            self.log_reader.abort()

        context.add_callback(on_rpc_done)
        log_output = self.log_reader.read(request.filename)

        try:
            for data in log_output:
                yield openocd_pb2.LogStreamResponse(data=data)
        except:
            _LOGGER.info("Cancelling RPC LogStreamOpen.")
            context.cancel()

        _LOGGER.debug("Regained servicer thread.")

    def ProgramDevice(self, request_iterator, context):
        _LOGGER.info("StartDebug called.")
        stop_event = threading.Event()
        def on_rpc_done():
            _LOGGER.debug("Attempting to regain servicer thread.")
            stop_event.set()
        context.add_callback(on_rpc_done)

        tmp = tempfile.NamedTemporaryFile()
        write_stream_to_file(tmp.name, request_iterator)
        log_output = openocd_cmd(self.config['cmd_program'].format(tmp.name))

        try:
            for data in log_output:
                yield openocd_pb2.LogStreamResponse(data=data)
        except:
            _LOGGER.info("Cancelling RPC RunOpenOcd.")
            context.cancel()

        _LOGGER.debug("Regained servicer thread.")

    def ResetDevice(self, request, context):
        _LOGGER.info("ResetDevice called")
        log_output = openocd_cmd(self.config['cmd_reset'])
        try:
            for data in log_output:
                yield openocd_pb2.LogStreamResponse(data=data)
        except:
            _LOGGER.info("Cancelling RPC RunOpenOcd.")
            context.cancel()

        _LOGGER.debug("Regained servicer thread.")

    def StartDebug(self, request, context):
        openocd_start_debug(self.config['cmd_debug'])
        _LOGGER.info("StartDebug called.")
        return openocd_pb2.void()

    def StopDebug(self, request, context):
        openocd_terminate()
        _LOGGER.info("StopDebug called.")
        return openocd_pb2.void()

class SignatureValidationInterceptor(grpc.ServerInterceptor):

    def __init__(self, auth):
        self.auth_key = auth

        def abort(ignored_request, context):
            context.abort(grpc.StatusCode.UNAUTHENTICATED, 'Invalid signature')

        self._abortion = grpc.unary_unary_rpc_method_handler(abort)

    def intercept_service(self, continuation, handler_call_details):
        method_name = handler_call_details.method.split('/')[-1]
        expected_metadata = (self.auth_key, method_name[::-1])
        if expected_metadata in handler_call_details.invocation_metadata:
            return continuation(handler_call_details)
        else:
            return self._abortion


def _running_server(config):
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=2), maximum_concurrent_rpcs=1)
    openocd_pb2_grpc.add_OpenOcdServicer_to_server(OpenOcd(config), server)
    actual_port = server.add_insecure_port(config['bindto'])
    server.start()
    return server


def _running_tls_server(config, auth):
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=2), maximum_concurrent_rpcs=1,
                         interceptors=(SignatureValidationInterceptor(auth),))

    openocd_pb2_grpc.add_OpenOcdServicer_to_server(OpenOcd(config), server)

    server_credentials = grpc.ssl_server_credentials(((
        _credentials.SERVER_CERTIFICATE_KEY,
        _credentials.SERVER_CERTIFICATE,
    ),))

    port = server.add_secure_port(config['bindto'], server_credentials)
    server.start()
    return server


def main():
    parser = argparse.ArgumentParser(description = 'oocd-rpcd')
    parser.add_argument(dest = 'config_file', nargs = '?', metavar = 'CONFIG', help = 'configuration file')
    args = parser.parse_args()
    parser = ConfigParser()

    if args.config_file == None or not Path(args.config_file).exists():
        raise ConfigException("Error: Missing configuration file.")

    parser.read(args.config_file)
    level_types = {'DEBUG': logging.DEBUG, 'INFO': logging.INFO,
                 'WARNING': logging.WARNING, 'ERROR': logging.ERROR, 'CRITICAL': logging.CRITICAL}

    if parser.has_section('log'):
        config = parser['log']
        loglevel = logging.ERROR
        if 'level' in config:
            if not config['level'] in level_types:
                raise ConfigException("Error: Invalid log level specified.")
            loglevel = level_types[config['level']]
        if 'file' in config:
            logging.basicConfig(filename=config['file'], encoding='utf-8', level=loglevel)
        else:
            logging.basicConfig()

    config = parser['DEFAULT']
    if 'tls_mode' in config and config['tls_mode'] == 'disabled':
        server = _running_server(config)
    else:
        if not 'cert_auth_key' in config:
            _LOGGER.error("'cert_auth_key' not specified.")
            os.exit(1)
        _credentials.load_certificates(config)
        server = _running_tls_server(config, config['cert_auth_key'])
    server.wait_for_termination()


if __name__ == "__main__":
    main()






# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import oocd_tool.openocd_pb2 as openocd__pb2


class OpenOcdStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ProgramDevice = channel.stream_stream(
                '/rpi.OpenOcd/ProgramDevice',
                request_serializer=openocd__pb2.ProgramRequest.SerializeToString,
                response_deserializer=openocd__pb2.LogStreamResponse.FromString,
                )
        self.ResetDevice = channel.unary_stream(
                '/rpi.OpenOcd/ResetDevice',
                request_serializer=openocd__pb2.void.SerializeToString,
                response_deserializer=openocd__pb2.LogStreamResponse.FromString,
                )
        self.StartDebug = channel.unary_unary(
                '/rpi.OpenOcd/StartDebug',
                request_serializer=openocd__pb2.void.SerializeToString,
                response_deserializer=openocd__pb2.void.FromString,
                )
        self.StopDebug = channel.unary_unary(
                '/rpi.OpenOcd/StopDebug',
                request_serializer=openocd__pb2.void.SerializeToString,
                response_deserializer=openocd__pb2.void.FromString,
                )
        self.LogStreamCreate = channel.unary_stream(
                '/rpi.OpenOcd/LogStreamCreate',
                request_serializer=openocd__pb2.LogStreamRequest.SerializeToString,
                response_deserializer=openocd__pb2.LogStreamResponse.FromString,
                )


class OpenOcdServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ProgramDevice(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartDebug(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopDebug(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LogStreamCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OpenOcdServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ProgramDevice': grpc.stream_stream_rpc_method_handler(
                    servicer.ProgramDevice,
                    request_deserializer=openocd__pb2.ProgramRequest.FromString,
                    response_serializer=openocd__pb2.LogStreamResponse.SerializeToString,
            ),
            'ResetDevice': grpc.unary_stream_rpc_method_handler(
                    servicer.ResetDevice,
                    request_deserializer=openocd__pb2.void.FromString,
                    response_serializer=openocd__pb2.LogStreamResponse.SerializeToString,
            ),
            'StartDebug': grpc.unary_unary_rpc_method_handler(
                    servicer.StartDebug,
                    request_deserializer=openocd__pb2.void.FromString,
                    response_serializer=openocd__pb2.void.SerializeToString,
            ),
            'StopDebug': grpc.unary_unary_rpc_method_handler(
                    servicer.StopDebug,
                    request_deserializer=openocd__pb2.void.FromString,
                    response_serializer=openocd__pb2.void.SerializeToString,
            ),
            'LogStreamCreate': grpc.unary_stream_rpc_method_handler(
                    servicer.LogStreamCreate,
                    request_deserializer=openocd__pb2.LogStreamRequest.FromString,
                    response_serializer=openocd__pb2.LogStreamResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rpi.OpenOcd', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class OpenOcd(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ProgramDevice(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/rpi.OpenOcd/ProgramDevice',
            openocd__pb2.ProgramRequest.SerializeToString,
            openocd__pb2.LogStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rpi.OpenOcd/ResetDevice',
            openocd__pb2.void.SerializeToString,
            openocd__pb2.LogStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartDebug(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpi.OpenOcd/StartDebug',
            openocd__pb2.void.SerializeToString,
            openocd__pb2.void.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopDebug(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rpi.OpenOcd/StopDebug',
            openocd__pb2.void.SerializeToString,
            openocd__pb2.void.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LogStreamCreate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rpi.OpenOcd/LogStreamCreate',
            openocd__pb2.LogStreamRequest.SerializeToString,
            openocd__pb2.LogStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from pathlib import Path

_GDBINIT = """tui enable
layout split
focus cmd
set print pretty
set print asm-demangle on
set mem inaccessible-by-default off
set pagination off
compare-sections
b main
"""

_OPENOCD_GDBINIT = """define restart
  mon reset halt
end

define rerun
  mon reset halt
  c
end
"""

_OPENOCD = """bindto 0.0.0.0
source [find interface/cmsis-dap.cfg]

transport select swd
source [find target/stm32f4x.cfg]
reset_config none

proc itm_log { OUTPUT F_CPU {BAUDRATE 2000000} } {
	tpiu create itm.tpiu -dap [dap names] -ap-num 0 -protocol uart
	itm.tpiu configure -traceclk $F_CPU -pin-freq $BAUDRATE -output $OUTPUT
	itm.tpiu enable
	tpiu init
	itm port 0 on
}

proc program_device { SOURCE } {
	program $SOURCE verify
	reset run
	shutdown
}

proc reset_device { } {
	reset run
	shutdown
}

init
"""

_OCD_TOOL = """[DEFAULT]
config_path: @CONFIG@
# gdb defaults
config.1: gdbinit
config.2: openocd_gdbinit
gdb_executable: arm-none-eabi-gdb-py
gdb_args: -ex "target extended-remote localhost:3333" -x @config.1@ -x @config.2@ @ELFFILE@
openocd_remote: localhost:50051
#tls_mode: disabled

# TLS uses buildin demo certificate if none specified.
# use 'examples/gen_certificates.sh -cn <hostname>' to generate new certificates. See README.md
cert_auth_key: my-secret-key
#root_ca: <filepath>

# User sections
[program]
openocd_args: program @ELFFILE@
mode: openocd

[reset]
openocd_args: reset
mode: openocd

[log]
openocd_args: logstream /tmp/test.log
mode: openocd

[gdb]
mode: gdb

[gui]
gdb_executable: gdbgui
gdb_args: '--gdb-cmd=${DEFAULT:gdb_executable} ${DEFAULT:gdb_args}'
mode: gdb
"""

_OCD_RPCD = """[DEFAULT]
bindto: 0.0.0.0:50051
cmd_program: openocd -f /home/ocd/.oocd-tool/openocd.cfg -c "program_device {}"
cmd_reset: openocd -f /home/ocd/.oocd-tool/openocd.cfg -c "reset_device"
cmd_debug: /usr/bin/openocd -f /home/ocd/.oocd-tool/openocd.cfg
#
# TLS uses buildin demo certificate if none specified
# use 'examples/gen_certificates.sh -cn <hostname>' to generate new certificates. See README.md
cert_auth_key: my-secret-key
#server_key: <filepath>
#server_cert: <filepath>

[log]
file: /tmp/ocd-rpcd.log
level: ERROR
"""

_OCD_RPCD_SERVICE = """[Unit]
Description=OpenOCD gRPC Service
After=network.target

[Service]
# Bug in gRPC version 1.42 environment variable can be delete in future versions.
Environment="LD_PRELOAD=/usr/lib/gcc/arm-linux-gnueabihf/10/libatomic.so"
ExecStart=/home/ocd/.local/bin/oocd-rpcd /home/ocd/.oocd-tool/oocd-rpcd.cfg
User=ocd
KillMode=process
Restart=always
RestartSec=10
Type=simple

[Install]
WantedBy=multi-user.target
"""

def create_default_config(path):
    Path(path).mkdir()
    with Path(path, 'gdbinit').open(mode='w') as f: f.write(_GDBINIT)
    with Path(path, 'openocd_gdbinit').open(mode='w') as f: f.write(_OPENOCD_GDBINIT)
    with Path(path, 'openocd.cfg').open(mode='w') as f: f.write(_OPENOCD)
    with Path(path, 'oocd-tool.cfg').open(mode='w') as f: f.write(_OCD_TOOL)
    with Path(path, 'oocd-rpcd.cfg').open(mode='w') as f: f.write(_OCD_RPCD)
    with Path(path, 'oocd-rpcd.service').open(mode='w') as f: f.write(_OCD_RPCD_SERVICE)



#!/usr/bin/python
#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import os
import psutil
import platform
import subprocess
from psutil import Process
from pathlib import PurePath, Path

class ConfigException(Exception):
    def __init__(self, message):
        self.message = message

class ProcessException(Exception):
    def __init__(self, message):
        self.message = message

class BackgroundProcess:
    def __init__(self, command, args, visible):
        self.cmd = command
        kwargs = { 'creationflags': subprocess.CREATE_NEW_PROCESS_GROUP } if 'Windows' in platform.platform() else {}
        redir = None if visible else subprocess.DEVNULL
        self._proc = subprocess.Popen(command + ' ' + args, stderr=redir, start_new_session=True, cwd=os.getcwd(), shell=True, **kwargs)

    def wait(self):
        self._proc.communicate()
        if self._proc.returncode != 0:
            raise ProcessException('Error: {} returncode: {}'.format(self.cmd, self._proc.returncode))

    def terminate(self):
        self._proc.terminate()
        if self._proc.returncode != None and self._proc.returncode > 1:
            raise ProcessException('Error: {} returncode: {}'.format(self.cmd, self._proc.returncode))

    def is_running(self):
        return True if self._proc.poll() == None else False

    def returncode(self):
        return self._proc.returncode

class BlockingProcess:
    def __init__(self, command, args):
        self.cmd = command
        _proc = subprocess.run(command + ' ' + args, cwd=os.getcwd(), shell=True)
        if _proc.returncode != 0:
            raise ProcessException('Error: {} returncode: {}'.format(self.cmd, _proc.returncode))


def is_process_running(name):
    for proc in psutil.process_iter(['pid', 'name']):
        try:
            if (proc.name() == name):
                return True, proc.pid
        except (psutil.NoSuchProcess, psutil.AccessDenied):
            pass
    return False, 0


def terminate(proc):
    if proc != None:
        if proc.is_running():
            proc.terminate()



#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import os
import signal
import psutil
import subprocess
import logging
import platform
from time import sleep

_LOGGER = logging.getLogger(__name__)

class ConfigException(Exception):
    def __init__(self, message):
        self.message = message

def openocd_cmd(cmd):
    killall("openocd")
    proc = subprocess.Popen(cmd, stderr=subprocess.PIPE, universal_newlines=True, cwd=os.getcwd(), shell=True)
    for ln in iter(proc.stderr.readline, ""):
        yield ln
    proc.stderr.close()
    ret = proc.wait()
    if ret:
        _LOGGER.error("openocd_program failed: '{}', returncode: {}".format(cmd, ret))
        raise subprocess.CalledProcessError(ret, cmd)

#def openocd_cmd(cmd):
#    killall("openocd")
#    proc = subprocess.run(cmd, cwd=os.getcwd(), shell=True)
#    if proc.returncode != 0:
#        _LOGGER.error("openocd_cmd failed: '{}', returncode: {}".format(cmd, proc.returncode))
#        raise subprocess.CalledProcessError(proc.returncode, cmd)

def openocd_start_debug(cmd):
    killall("openocd")
    proc = subprocess.Popen(cmd, cwd=os.getcwd(), shell=True)
    sleep(0.1)
    ret = proc.poll()
    if ret != None:
        _LOGGER.error("openocd_start_debug failed: '{}', returncode: {}".format(cmd, ret))
        raise subprocess.CalledProcessError(ret, 'openocd_start_debug')


def openocd_terminate():
    killall("openocd")


def killall(name):
    l = process_pid_list(name)
    for pid in l:
        os.kill(pid, signal.SIGTERM)


def write_stream_to_file(filename, request_iterator):
        file = open(filename, "wb")
        for request in request_iterator:
            file.write(request.data)
        file.close()


def process_pid_list(name):
    res = []
    for proc in psutil.process_iter(['pid', 'name']):
        try:
            if (proc.name() == name):
                res.append(proc.pid)
        except (psutil.NoSuchProcess, psutil.AccessDenied):
            pass
    return res


class LogReader:
    def __init__(self):
      self.done = False
      pass

    def read(self, filename):
      with open(filename, 'r') as file:
          line = ''
          while not self.done:
              tmp = file.readline()
              if tmp != "":
                  line += tmp
                  if line.endswith("\n"):
                      yield line
                      line = ''
              else:
                  sleep(0.1)

    def abort(self):
        self.done = True


# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

def _load_credential_from_file(filepath):
    real_path = os.path.join(os.path.dirname(__file__), filepath)
    with open(real_path, 'rb') as f:
        return f.read()


SERVER_CERTIFICATE = _load_credential_from_file('credentials/localhost.crt')
SERVER_CERTIFICATE_KEY = _load_credential_from_file('credentials/localhost.key')
ROOT_CERTIFICATE = _load_credential_from_file('credentials/root.crt')


def load_certificates(cert):
    if 'root_ca' in cert:
        global ROOT_CERTIFICATE
        ROOT_CERTIFICATE = _load_credential_from_file(cert['root_ca'])
    if 'server_cert' in cert:
        global SERVER_CERTIFICATE
        SERVER_CERTIFICATE = _load_credential_from_file(cert['server_cert'])
    if 'server_key' in cert:
        global SERVER_CERTIFICATE_KEY
        SERVER_CERTIFICATE_KEY = _load_credential_from_file(cert['server_key'])





#!/usr/bin/python3
#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import re
import sys
import signal
import argparse
import tempfile
import oocd_tool.rpc_client as rpc_client

from time import sleep
from configparser import ConfigParser, ExtendedInterpolation
from pathlib import PurePath, Path
from oocd_tool.process import *

def signal_handler(sig, frame):
    pass

def error_exit(message):
    sys.stderr.write(message + '\n')
    sys.exit(1)

def get_config_value(config, key, message):
    if not key in config:
        raise ConfigException(message)
    return config[key]

def parse_config(file, section):
    parser = ConfigParser(interpolation=ExtendedInterpolation())
    parser.read(file)
    if not parser.has_section(section):
        raise ConfigException("Error: invalid section: {}".format(section))
    items = parser.items(section)
    for node in items:
        yield node


def translate(nodes, **options):
    class Res:
        has_tmpfile = False
        nodes = {}
        files = {}

    tmpcnt = 0
    for key, value in nodes:
        value = re.sub(r'@CONFIG@', options['config'], value)
        value = re.sub(r'@ELFFILE@', options['elf'], value)
        if value.find('@TMPFILE@') != -1:
            if tmpcnt == 0:
                Res.tmpfile = tempfile.NamedTemporaryFile()
                value = re.sub(r'@TMPFILE@', Res.tmpfile.name, value)
                Res.has_tmpfile = True
                tmpcnt += 1
            elif tmpcnt == 1:
                value = re.sub(r'@TMPFILE@', Res.tmpfile.name, value)
            else:
                raise ConfigException("Error: @TMPFILE@ may only be used in pairs once.")
        if value.find('@FCPU@') != -1:
            if options['fcpu'] == None: raise ConfigException('Error: --fcpu is missing.')
            value = re.sub(r'@FCPU@', str(options['fcpu']), value)
        if re.match('config\..+', key):
            Res.files['@{}@'.format(key)] = value
        else:
            Res.nodes[key] = value

    # Append path to config keys there only contains filename
    path = Res.nodes['config_path'] if 'config_path' in Res.nodes else options['config_path']
    for key, file in Res.files.items():
        Res.files[key] = file if file.count('/') != 0 else str(PurePath(PurePath(path), PurePath(file)))

    # Replace @config@ tags with real path
    expr = re.compile(r'@config\.[a-z0-9]+@')
    for key, item in Res.nodes.items():
        tags = expr.findall(item)
        for tag in tags:
            item = item.replace(tag, Res.files[tag])
        Res.nodes[key] = item
    return Res


def check_mandatory_keys(config, key_list):
    for key in key_list:
        if not key in config:
            if not (key == 'openocd_executable' and 'openocd_remote' in config):
                raise ConfigException('Error: missing configuration entry: {}'.format(key))


def check_executable(file):
    import shutil
    if shutil.which(file) == None:
        raise ConfigException('Error: executable not found: {}'.format(file))


def validate_configuration(config, section):
    if not 'mode' in config or not config['mode'] in ['gdb_openocd', 'openocd', 'gdb']:
        raise ConfigException('Error: mode not specified in section: [{}]'.format(section))
    if config['mode'] == 'gdb':
        check_mandatory_keys(config, ['gdb_executable', 'gdb_args'])
        check_executable(config['gdb_executable'])
    if config['mode'] != 'gdb':
        check_mandatory_keys(config, ['openocd_executable', 'openocd_args'])
        if 'openocd_remote' not in config:
            check_executable(config['openocd_executable'])


def validate_files(files):
    for key, file in files.items():
        if not Path(file).is_file():
            raise ConfigException('Error: file not found: {}'.format(file))


def create_default_config():
    path = Path(Path.home(), ".oocd-tool")
    if (not path.exists()):
        from .configs import create_default_config
        create_default_config(path)

def default_config_file():
    path = Path(Path(Path.home(), ".oocd-tool"), Path('oocd-tool.cfg'))
    if (not path.exists()):
        raise ConfigException("Error: default config '{}' not found.".format(path))
    return path


def raise_if_running(filename):
    running, pid = is_process_running(filename)
    if running: raise ProcessException('Error: openocd is already runnning with pid: {}'.format(pid))


def run_openocd_remote(rpc, args):
    n = args.find(' ')
    cmd = args if n == -1 else args[0 : n]
    if cmd == 'program' and n != -1:
        stream = rpc.program_device(args[len(cmd) + 1:])
        for line in stream:
            print(line)
    elif cmd == 'reset':
        stream = rpc.reset_device()
        for line in stream:
            print(line)
    elif cmd == 'logstream' and n != -1:
        stream = rpc.log_stream_create(args[len(cmd) + 1:])
        for line in stream:
            print(line)
    else:
        raise ConfigException('Error: invalid ocdrpc mode: {}'.format(args))


class LocalExecuteInterface:
    def __init__(self):
        self.ocd = None

    def spawn_process(self, cfg):
        proc = BackgroundProcess(cfg['spawn_process'], '', True)

    def debug_spawned_openocd(self, cfg):
        raise_if_running(cfg['openocd_executable'])
        self.ocd = BackgroundProcess(cfg['openocd_executable'], cfg['openocd_args'], False)
        sleep(0.1)
        if not self.ocd.is_running():
            raise ProcessException('Error: openocd prematurely exited with code: {}'.
                                   format(self.ocd.returncode()))
        BlockingProcess(cfg['gdb_executable'], cfg['gdb_args'])
        self.ocd.terminate()

    def debug(self, cfg):
        raise_if_running(cfg['openocd_executable'])
        BlockingProcess(cfg['gdb_executable'], cfg['gdb_args'])

    def openocd_only(self, cfg):
        raise_if_running(cfg['openocd_executable'])
        self.ocd = BackgroundProcess(cfg['openocd_executable'], cfg['openocd_args'], True)
        self.ocd.wait()
        pass

    def __del__(self):
        terminate(self.ocd)


class RemoteExecuteInterface:
    def __init__(self, channel, auth_key):
        self.channel = channel
        self.auth_key = auth_key

    def spawn_process(self):
        # TODO
        raise ConfigException("Not implemented yet.")

    def debug_spawned_openocd(self):
        raise ConfigException("Invalid mode configured.")

    def debug(self, cfg):
        rpc = rpc_client.ClientChannel(cfg['openocd_remote'], self.channel, self.auth_key)
        with rpc.debug_device():
            BlockingProcess(cfg['gdb_executable'], cfg['gdb_args'])

    def openocd_only(self, cfg):
        rpc = rpc_client.ClientChannel(cfg['openocd_remote'], self.channel, self.auth_key)
        run_openocd_remote(rpc, cfg['openocd_args'])


def execute(cfg):
    auth_key = ""
    channel = rpc_client.secure_channel
    exec = None

    if 'openocd_remote' in cfg:
        if 'tls_mode' in cfg and cfg['tls_mode'] == 'disabled':
            channel = rpc_client.insecure_channel
        else:
            auth_key = get_config_value(cfg, 'cert_auth_key', "Error: 'cert_signature_key' not specified.")
        exec = RemoteExecuteInterface(channel, auth_key)
    else:
        exec = LocalExecuteInterface()

    if 'spawn_process' in cfg:
        exec.spawn_process(cfg)

    if cfg['mode'] == 'gdb_openocd':
        signal.signal(signal.SIGINT, signal_handler)
        exec.debug_spawned_openocd(cfg)

    elif cfg['mode'] == 'openocd':
        exec.openocd_only(cfg)

    elif cfg['mode'] == 'log':
        #TODO
        raise ConfigException("Not implemented yet.")

    elif cfg['mode'] == 'gdb':
        signal.signal(signal.SIGINT, signal_handler)
        exec.debug(cfg)


def main():
    parser = argparse.ArgumentParser(description = 'oocd-tool')
    parser.add_argument(dest = 'section', nargs = '?', metavar='SECTION', help = 'section in config file to run')
    parser.add_argument(dest = 'source', nargs = '?', metavar='ELF', help = 'target elf file')
    parser.add_argument('-c', dest = 'config', nargs = '?', metavar='CONFIG', help = 'config file')
    parser.add_argument('--fcpu', action='store', type=int, metavar='FREQ', help = 'cpu clock (used with itm logging)')
    parser.add_argument('-d', action='store_true', help = 'dry run')
    args = parser.parse_args()

    create_default_config()
    if args.config == None:
        args.config = default_config_file()

    if args.section == None:
        error_exit("Section not specified.")
    if args.source != None and not Path(args.source).is_file():
        error_exit("ELF file does not exists.")
    if not Path(args.config).is_file():
        error_exit("Error cannot open config file: {}.".format(args.config))
    if args.source == None: args.source = ''

    # regex fails with windows path's. as_posix() used as workarround
    config_path = PurePath(args.config).parent.as_posix()

    pc = parse_config(args.config, args.section)
    result = translate(pc, config=config_path, elf=args.source, fcpu=args.fcpu)
    cfg = result.nodes

    # dry run
    if args.d:
        print('')
        if cfg['mode'] in ['gdb_openocd', 'gdb']: print('gdb: {} {}\n'.format(cfg['gdb_executable'], cfg['gdb_args']))
        if cfg['mode'] in ['gdb_openocd', 'openocd']: print('openocd: {} {}\n'.format(cfg['openocd_executable'], cfg['openocd_args']))
        if 'spawn_process' in cfg: print('spawn: {}\n'.format(cfg['spawn_process']))
        sys.exit(0)

    validate_configuration(cfg, args.section)
    validate_files(result.files)
    rpc_client.load_certificates(cfg)
    execute(cfg)


if __name__ == "__main__":
    main()




#
# Copyright (C) 2021 Jacob Schultz Andersen schultz.jacob@gmail.com
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import sys
import grpc
import signal
import contextlib
import oocd_tool.openocd_pb2 as openocd_pb2
import oocd_tool.openocd_pb2_grpc as openocd_pb2_grpc
import oocd_tool._credentials as _credentials


def _setup_cancel_request(generator):
    def cancel_request(unused_signum, unused_frame):
        generator.cancel()
        sys.exit(0)

    signal.signal(signal.SIGINT, cancel_request)

class AuthGateway(grpc.AuthMetadataPlugin):
    def __init__(self, auth_key):
        self.auth_key = auth_key

    def __call__(self, context, callback):
        signature = context.method_name[::-1]
        callback(((self.auth_key, signature),), None)

@contextlib.contextmanager
def insecure_channel(addr, unused_signatur):
    yield grpc.insecure_channel(addr)

@contextlib.contextmanager
def secure_channel(addr, auth):
    call_credentials = grpc.metadata_call_credentials(
        AuthGateway(auth), name='auth gateway')
    channel_credential = grpc.ssl_channel_credentials(
        _credentials.ROOT_CERTIFICATE)
    composite_credentials = grpc.composite_channel_credentials(
        channel_credential, call_credentials)

    channel = grpc.secure_channel(addr, composite_credentials)
    yield channel

def load_certificates(config):
    _credentials.load_certificates(config)

class ClientChannel:
    def __init__(self, host, channel, auth):
        self._host = host
        self._channel_type = channel
        self._auth_key = auth

    def is_secure(self):
        return self._channel_type == secure_channel

    def log_stream_create(self, file):
        with self._channel_type(self._host, self._auth_key) as channel:
            stub = openocd_pb2_grpc.OpenOcdStub(channel)

            result_generator = stub.LogStreamCreate(openocd_pb2.LogStreamRequest(filename=file))
            _setup_cancel_request(result_generator)

            for result in result_generator:
                yield result.data.strip()

    def program_device(self, file):
        with self._channel_type(self._host, self._auth_key) as channel:
            stub = openocd_pb2_grpc.OpenOcdStub(channel)

            def file_reader(filename):
                with open(filename, 'rb') as file:
                    while chunk := file.read(2048):
                        request = openocd_pb2.ProgramRequest(data=chunk)
                        yield request

            result_generator = stub.ProgramDevice(file_reader(file))
            _setup_cancel_request(result_generator)

            for result in result_generator:
               yield result.data.strip()

    def reset_device(self):
        with self._channel_type(self._host, self._auth_key) as channel:
            stub = openocd_pb2_grpc.OpenOcdStub(channel)
            result_generator = stub.ResetDevice(openocd_pb2.void())
            _setup_cancel_request(result_generator)

            for result in result_generator:
               yield result.data.strip()


    @contextlib.contextmanager
    def debug_device(self):
        with self._channel_type(self._host, self._auth_key) as channel:
            stub = openocd_pb2_grpc.OpenOcdStub(channel)
        try:
            stub.StartDebug(openocd_pb2.void())
            yield
        finally:
            stub.StopDebug(openocd_pb2.void())





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="oocd-tool",
    version="0.2.0",
    author="Jacob Schultz Andersen",
    author_email="schultz.jacob@gmail.com",
    description="A flexible configuration and remote contol tool for openocd.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jasa/oocd-tool.git",
    project_urls={
        "Bug Tracker": "https://github.com/jasa/oocd-tool/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
    ],
    install_requires = [
         "setuptools>=42",
         "psutil>=5",
         "grpcio>=1.41",
         "grpcio-tools>=1.41"
    ],
    keywords='arm gdb cortex cortex-m trace microcontroller',
    packages = setuptools.find_packages(),
    python_requires=">=3.6",
    scripts = ['bin/oocd-tool', 'bin/oocd-rpcd'],
    entry_points = {
        "console_scripts": [
            "oocd_tool = oocd_tool.main:main",
        ],
    },
    include_package_data=True,
    package_data={ "": ["oocd_tool/credentials/*"] },
)
